@@ -51,6 +51,7 @@
 #include <miktex/Core/CommandLineBuilder>
 #include <miktex/Core/StreamReader>
 
+#include <miktex/Util/PathNameUtil>
 #include <miktex/Util/Tokenizer>
 
 #include "internal.h"
@@ -206,6 +207,15 @@ void unxProcess::Create()
 {
   MIKTEX_EXPECT(!startinfo.FileName.empty());
 
+  shared_ptr<SessionImpl> session = SessionImpl::TryGetSession();
+
+  PathName fileName;
+
+  if (PathNameUtil::IsAbsolutePath(startinfo.FileName) || session == nullptr || !session->FindFile(startinfo.FileName, FileType::EXE, fileName))
+  {
+    fileName = startinfo.FileName;
+  }
+
   Argv argv(startinfo.Arguments.empty() ? vector<string>{ PathName(startinfo.FileName).GetFileNameWithoutExtension().ToString() } : startinfo.Arguments);
 
   Pipe pipeStdout;
@@ -248,8 +258,6 @@ void unxProcess::Create()
     pipeStdin.Create();
   }
 
-  shared_ptr<SessionImpl> session = SessionImpl::TryGetSession();
-
   if (session != nullptr)
   {
     session->UnloadFilenameDatabase();
@@ -336,9 +344,9 @@ void unxProcess::Create()
             args += ", ";
           }
         }
-        session->trace_process->WriteLine("core", TraceLevel::Info, fmt::format("execv: \"{0}\", [ {1} ]", startinfo.FileName, args));
+        session->trace_process->WriteLine("core", TraceLevel::Info, fmt::format("execv: \"{0}\", [ {1} ]", fileName, args));
       }
-      execv(startinfo.FileName.c_str(), const_cast<char*const*>(argv.GetArgv()));
+      execv(fileName.GetData(), const_cast<char*const*>(argv.GetArgv()));
       perror("execv failed");
     }
     catch (const exception&)
@@ -485,14 +493,6 @@ void unxProcess::WaitForExit()
     {
       session->trace_process->WriteLine("core", fmt::format("process {0} terminated due to signal {1}", pid, WTERMSIG(status)));
     }
-    else if (WIFSTOPPED(status) != 0)
-    {
-      session->trace_process->WriteLine("core", fmt::format("process {0} stopped due to signal {1}", pid, WSTOPSIG(status)));
-    }
-    else if (WIFCONTINUED(status) != 0)
-    {
-      session->trace_process->WriteLine("core", fmt::format("process {0} continued", pid));
-    }
   }
 }
 
@@ -522,19 +522,32 @@ bool unxProcess::WaitForExit(int milliseconds)
   return false;
 }
 
+ProcessExitStatus unxProcess::get_ExitStatus() const
+{
+  if (WIFEXITED(status) != 0)
+  {
+    return ProcessExitStatus::Exited;
+  }
+  else if (WIFSIGNALED(status) != 0)
+  {
+    return ProcessExitStatus::Signaled;
+  }
+  else if (WIFSTOPPED(status) != 0)
+  {
+    return ProcessExitStatus::Other;
+  }
+}
+
+
 int unxProcess::get_ExitCode() const
 {
   if (WIFEXITED(status) != 0)
   {
     return WEXITSTATUS(status);
   }
-  else if (WIFSIGNALED(status) != 0)
-  {
-    MIKTEX_FATAL_ERROR_2(T_("Process terminated due to a signal."), "fileName", startinfo.FileName, "signal", std::to_string(WTERMSIG(status)));
-  }
   else
   {
-    MIKTEX_UNEXPECTED();
+    MIKTEX_FATAL_ERROR_2(T_("Process terminated unexpectedly."), "fileName", startinfo.FileName, "exitStatus", std::to_string(status));
   }
 }
 
@@ -744,7 +757,7 @@ ProcessInfo unxProcess::GetProcessInfo()
         processInfo.status = ProcessStatus::Sleeping;
         break;
       case 'T':
-        processInfo.status = ProcessStatus::Stoped;
+        processInfo.status = ProcessStatus::Stopped;
         break;
       case 'Z':
         processInfo.status = ProcessStatus::Zombie;
@@ -775,7 +788,7 @@ ProcessInfo unxProcess::GetProcessInfo()
       processInfo.status = ProcessStatus::Sleeping;
       break;
     case SSTOP:
-      processInfo.status = ProcessStatus::Stoped;
+      processInfo.status = ProcessStatus::Stopped;
       break;
     case SZOMB:
       processInfo.status = ProcessStatus::Zombie;
@@ -804,7 +817,7 @@ ProcessInfo unxProcess::GetProcessInfo()
           processInfo.status = ProcessStatus::Sleeping;
           break;
         case SSTOP:
-          processInfo.status = ProcessStatus::Stoped;
+          processInfo.status = ProcessStatus::Stopped;
           break;
         case SZOMB:
           processInfo.status = ProcessStatus::Zombie;
