@@ -28,6 +28,7 @@
 #include <thread>
 
 #include <miktex/Core/File>
+#include <miktex/Core/FileStream>
 #include <miktex/Core/LockFile>
 #include <miktex/Core/PathName>
 
@@ -37,14 +38,17 @@ using namespace chrono_literals;
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Test;
 
-BEGIN_TEST_SCRIPT("lockfile-1-2");
+BEGIN_TEST_SCRIPT("lockfile-1-3");
 
 BEGIN_TEST_FUNCTION(1);
 {
-  unique_ptr<MiKTeX::Core::LockFile> lockFile = LockFile::Create(PathName("lockfile-1-2"));
-  TEST(lockFile->TryLock(0s));
-  lockFile.release();
+  FileStream writer(File::Open(PathName("sharedfile-1"), FileMode::Create, FileAccess::Write));
+  TEST(File::TryLock(writer.GetFile(), File::LockType::Exclusive, 0ms));
+  writer.Write("01234", 5);
   this_thread::sleep_for(5s);
+  writer.Write("56789", 5);
+  fflush(writer.GetFile());
+  TESTX(File::Unlock(writer.GetFile()));
 }
 END_TEST_FUNCTION();
 
