@@ -561,6 +561,12 @@ inline char* concat(const char* s1, const char* s2)
 
 #define xgetcwd() miktex_xgetcwd()
 
+#define kpathsea_dir_p(kpse, fn) miktex_kpathsea_dir_p(kpse, fn)
+
+#if defined (KPSE_COMPAT_API)
+#  define dir_p(fn) kpathsea_dir_p(kpse_def, fn)
+#endif
+
 /// @}
 
 /* _________________________________________________________________________
@@ -1049,6 +1055,8 @@ MIKTEXKPSCEEAPI(int) miktex_kpathsea_bitmap_tolerance(kpathsea kpseInstance, dou
 
 MIKTEXKPSCEEAPI(char*) miktex_kpathsea_brace_expand(kpathsea kpseInstance, const char* path);
 
+MIKTEXKPSCEEAPI(boolean) miktex_kpathsea_dir_p(kpathsea kpse, string fn);
+
 MIKTEXKPSCEEAPI(char*) miktex_kpathsea_find_file(kpathsea kpseInstance, const char* fileName, kpse_file_format_type format, int mustExist);
 
 MIKTEXKPSCEEAPI(char**) miktex_kpathsea_find_file_generic(kpathsea kpseInstance, const char* fileName, kpse_file_format_type format, boolean mustExist, boolean all);
