@@ -90,7 +90,7 @@ int Wrapper::Run(int(*Main)(int argc, char* argv[]), int argc, char* argv[])
       Utils::SetEnvironmentString("MIKTEX_LOG_DIR", PathName(session->GetSpecialPath(SpecialPath::DataRoot)).AppendComponent(MIKTEX_PATH_MIKTEX_LOG_DIR).ToString());
       Utils::SetEnvironmentString("MIKTEX_LOG_NAME", "texworks");
       log4cxx::xml::DOMConfigurator::configure(xmlFileName.ToWideCharString());
-      LOG4CXX_INFO(logger, "starting: " << Utils::MakeProgramVersionString("MiKTeX TeXworks", MIKTEX_COMP_ORIG_VERSION_STR));
+      LOG4CXX_INFO(logger, "starting: " << Utils::MakeProgramVersionString("MiKTeX TeXworks", VersionNumber(MIKTEX_COMP_ORIG_VERSION_STR)));
       FlushPendingTraceMessages();
       isLog4cxxConfigured = true;
       string cmdline;
@@ -125,7 +125,7 @@ int Wrapper::Run(int(*Main)(int argc, char* argv[]), int argc, char* argv[])
   }
 }
 
-void Wrapper::Trace(const TraceCallback::TraceMessage& traceMessage)
+bool Wrapper::Trace(const TraceCallback::TraceMessage& traceMessage)
 {
   if (isLog4cxxConfigured)
   {
@@ -135,6 +135,7 @@ void Wrapper::Trace(const TraceCallback::TraceMessage& traceMessage)
   {
     pendingTraceMessages.push_back(traceMessage);
   }
+  return true;
 }
 
 void Wrapper::FlushPendingTraceMessages()
