@@ -1,23 +1,23 @@
 /************
-*
-*   This file is part of the vector graphics language Asymptote
-*   Copyright (C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince
-*                 http://asymptote.sourceforge.net
-*
-*   This program is free software: you can redistribute it and/or modify
-*   it under the terms of the GNU Lesser General Public License as published by
-*   the Free Software Foundation, either version 3 of the License, or
-*   (at your option) any later version.
-*
-*   This program is distributed in the hope that it will be useful,
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*   GNU Lesser General Public License for more details.
-*
-*   You should have received a copy of the GNU Lesser General Public License
-*   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*
-*************/
+ *
+ *   This file is part of the vector graphics language Asymptote
+ *   Copyright (C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince
+ *                 http://asymptote.sourceforge.net
+ *
+ *   This program is free software: you can redistribute it and/or modify
+ *   it under the terms of the GNU Lesser General Public License as published by
+ *   the Free Software Foundation, either version 3 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public License
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ *************/
 
 #ifdef __CYGWIN__
 #define _POSIX_C_SOURCE 200809L
@@ -59,6 +59,10 @@ using namespace settings;
 
 using interact::interactive;
 
+namespace gl {
+extern bool glexit;
+}
+
 namespace run {
 void purge();
 }
@@ -199,13 +203,17 @@ void *asymain(void *A)
   }
 #ifdef HAVE_GL
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
-  if(gl::glthread && !getSetting<bool>("offscreen")) {
+  if(gl::glthread) {
+#ifdef __MSDOS__ // Signals are unreliable in MSWindows
+    gl::glexit=true;
+#else
 #if defined(MIKTEX) && !defined(HAVE_PTHREAD)
     MiKTeX::Aymptote::exitRequested = true;
     gl::initSignal.notify_one();
 #else
     pthread_kill(gl::mainthread,SIGURG);
     pthread_join(gl::mainthread,NULL);
+#endif
 #endif
   }
 #endif
@@ -272,7 +280,7 @@ int main(int argc, char *argv[])
 #endif
   gl::glthread=usethreads ? getSetting<bool>("threads") : false;
 #if HAVE_PTHREAD
-
+#ifndef HAVE_LIBOSMESA
   if(gl::glthread) {
     pthread_t thread;
     try {
@@ -292,6 +300,7 @@ int main(int argc, char *argv[])
       outOfMemory();
     }
   }
+#endif
 #elif defined(MIKTEX)
   if (gl::glthread)
   {
