@@ -338,8 +338,10 @@ map<string, string> MakeFontMapApp::psADOBE;
 void MakeFontMapApp::ShowVersion()
 {
   cout
-    << Utils::MakeProgramVersionString(THE_NAME_OF_THE_GAME, VersionNumber(MIKTEX_MAJOR_VERSION, MIKTEX_MINOR_VERSION, MIKTEX_COMP_J2000_VERSION, 0)) << endl
-    << "Copyright (C) 2002-2020 Christian Schenk" << endl
+    << Utils::MakeProgramVersionString(THE_NAME_OF_THE_GAME, VersionNumber(MIKTEX_COMPONENT_VERSION_STR)) << endl
+    << endl
+    << MIKTEX_COMP_COPYRIGHT_STR << endl
+    << endl
     << "This is free software; see the source for copying conditions.  There is NO" << endl
     << "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
 }
