@@ -269,14 +269,12 @@ void init_shell_escape(void)
 
 #  if 0
 #  ifdef WIN32
-#if !defined(MIKTEX)
 static int char_needs_quote(int c)
 {
     /* special characters of cmd.exe */
     return (c == '&' || c == '|' || c == '%' || c == '<' ||
             c == '>' || c == ';' || c == ',' || c == '(' || c == ')');
 }
-#endif
 #  endif
 #  endif
 
@@ -299,6 +297,9 @@ static int Isspace(char c)
 
 int shell_cmd_is_allowed(const char *cmd, char **safecmd, char **cmdname)
 {
+#if defined(MIKTEX)
+  return miktex_emulate__shell_cmd_is_allowed(cmd, safecmd, cmdname);
+#else
     char **p;
     char *buf;
     char *c, *d;
@@ -418,10 +419,8 @@ int shell_cmd_is_allowed(const char *cmd, char **safecmd, char **cmdname)
                         return -1;
 #  if 0
 #  ifdef WIN32
-#if !defined(MIKTEX)
                     if (char_needs_quote(*s))
                         *d++ = '^';
-#endif
 #  endif
 #  endif
                     *d++ = *s++;
@@ -445,10 +444,8 @@ int shell_cmd_is_allowed(const char *cmd, char **safecmd, char **cmdname)
                 *d++ = QUOTE;
 #  if 0
 #  ifdef WIN32
-#if !defined(MIKTEX)
                 if (char_needs_quote(*s))
                     *d++ = '^';
-#endif
 #  endif
 #  endif
                 *d++ = *s++;
@@ -466,10 +463,8 @@ int shell_cmd_is_allowed(const char *cmd, char **safecmd, char **cmdname)
                 */
 #  if 0
 #  ifdef WIN32
-#if !defined(MIKTEX)
                 if (char_needs_quote(*s))
                     *d++ = '^';
-#endif
 #  endif
 #  endif
                 *d++ = *s++;
@@ -483,8 +478,6 @@ int shell_cmd_is_allowed(const char *cmd, char **safecmd, char **cmdname)
         }
         *d = '\0';
 #ifdef WIN32
-#if defined(MIKTEX)
-#else
         {
           char *p, *q, *r;
           p = *safecmd;
@@ -521,10 +514,10 @@ int shell_cmd_is_allowed(const char *cmd, char **safecmd, char **cmdname)
             }
           }
         }
-#endif
 #endif
     }
     return allow;
+#endif
 }
 
 #endif
@@ -616,27 +609,23 @@ main (int ac, string *av)
 
     lua_initialize(ac, av);
 
+#if !defined(MIKTEX)
 #  ifdef WIN32
     if (ac > 1) {
         char *pp;
-#if defined(MIKTEX_WINDOWS)
-        if ((strlen(av[ac - 1]) > 2) && isalpha(av[ac - 1][0]) && (av[ac - 1][1] == ':') && (av[ac - 1][2] == '\\' || av[ac - 1][2] == '/')) {
-#else
         if ((strlen(av[ac-1]) > 2) && isalpha(av[ac-1][0]) && (av[ac-1][1] == ':') && (av[ac-1][2] == '\\')) {
-#endif
             for (pp=av[ac-1]+2; *pp; pp++) {
-#if !defined(MIKTEX)
             if (IS_KANJI(pp)) {
                 pp++;
                 continue;
             }
-#endif
             if (*pp == '\\')
                 *pp = '/';
             }
         }
     }
 #  endif
+#endif
 
     /*
         Call the real main program.
@@ -906,7 +895,11 @@ boolean input_line(FILE * f)
     int i = EOF;
 
 #ifdef WIN32
+#if defined(MIKTEX)
+    if (!miktex_is_pipe(f) && fileno(f) != fileno(stdin)) {
+#else
     if (f != Poptr && fileno (f) != fileno (stdin)) {
+#endif
         long position = ftell (f);
         if (position == 0L) {
             /* Detect and skip Byte order marks.  */
